@@ -9,15 +9,14 @@
 """
 
 __author__ = "Ryan McGrath <ryan@venodesigns.net>"
-__version__ = "2.0.1"
+__version__ = "2.3.0"
 
 import urllib
 import re
-import time
+import warnings
 
 import requests
 from requests.auth import OAuth1
-import oauth2 as oauth
 
 try:
     from urlparse import parse_qsl
@@ -68,11 +67,6 @@ class TwythonError(Exception):
                          twitter_http_status_codes[error_code][1],
                          self.msg)
 
-        if error_code == 420:
-            raise TwythonRateLimitError(self.msg,
-                                        error_code,
-                                        retry_after=retry_after)
-
     def __str__(self):
         return repr(self.msg)
 
@@ -81,12 +75,7 @@ class TwythonAuthError(TwythonError):
     """ Raised when you try to access a protected resource and it fails due to
         some issue with your authentication.
     """
-    def __init__(self, msg, error_code=None):
-        self.msg = msg
-        self.error_code = error_code
-
-    def __str__(self):
-        return repr(self.msg)
+    pass
 
 
 class TwythonRateLimitError(TwythonError):
@@ -94,25 +83,23 @@ class TwythonRateLimitError(TwythonError):
         retry_wait_seconds is the number of seconds to wait before trying again.
     """
     def __init__(self, msg, error_code, retry_after=None):
+        TwythonError.__init__(self, msg, error_code=error_code)
         if isinstance(retry_after, int):
-            retry_after = int(retry_after)
-            self.msg = '%s (Retry after %s seconds)' % (msg, retry_after)
-
-    def __str__(self):
-        return repr(self.msg)
+            self.msg = '%s (Retry after %d seconds)' % (msg, retry_after)
 
 
 class Twython(object):
     def __init__(self, app_key=None, app_secret=None, oauth_token=None, oauth_token_secret=None, \
-                headers=None, callback_url=None, twitter_token=None, twitter_secret=None):
+                headers=None, callback_url=None, twitter_token=None, twitter_secret=None, proxies=None):
         """Instantiates an instance of Twython. Takes optional parameters for authentication and such (see below).
 
             :param app_key: (optional) Your applications key
             :param app_secret: (optional) Your applications secret key
-            :param oauth_token: (optional) Used with oauth_secret to make authenticated calls
-            :param oauth_secret: (optional) Used with oauth_token to make authenticated calls
+            :param oauth_token: (optional) Used with oauth_token_secret to make authenticated calls
+            :param oauth_token_secret: (optional) Used with oauth_token to make authenticated calls
             :param headers: (optional) Custom headers to send along with the request
             :param callback_url: (optional) If set, will overwrite the callback url set in your application
+            :param proxies: (optional) A dictionary of proxies, for example {"http":"proxy.example.org:8080", "https":"proxy.example.org:8081"}.
         """
 
         # Needed for hitting that there API.
@@ -135,9 +122,9 @@ class Twython(object):
         if oauth_token is not None:
             self.oauth_token = u'%s' % oauth_token
 
-        self.oauth_secret = None
+        self.oauth_token_secret = None
         if oauth_token_secret is not None:
-            self.oauth_secret = u'%s' % oauth_token_secret
+            self.oauth_token_secret = u'%s' % oauth_token_secret
 
         self.callback_url = callback_url
 
@@ -153,15 +140,15 @@ class Twython(object):
             self.auth = OAuth1(self.app_key, self.app_secret,
                                signature_type='auth_header')
 
-        if self.oauth_token is not None and self.oauth_secret is not None:
+        if self.oauth_token is not None and self.oauth_token_secret is not None:
             self.auth = OAuth1(self.app_key, self.app_secret,
-                               self.oauth_token, self.oauth_secret,
+                               self.oauth_token, self.oauth_token_secret,
                                signature_type='auth_header')
 
         if self.client is None:
             # If they don't do authentication, but still want to request
             # unprotected resources, we need an opener.
-            self.client = requests.session()
+            self.client = requests.session(proxies=proxies)
 
         # register available funcs to allow listing name when debugging.
         def setFunc(key):
@@ -183,27 +170,29 @@ class Twython(object):
         )
 
         method = fn['method'].lower()
-        if not method in ('get', 'post', 'delete'):
-            raise TwythonError('Method must be of GET, POST or DELETE')
+        if not method in ('get', 'post'):
+            raise TwythonError('Method must be of GET or POST')
 
         content = self._request(url, method=method, params=kwargs)
 
         return content
 
-    def _request(self, url, method='GET', params=None, api_call=None):
+    def _request(self, url, method='GET', params=None, files=None, api_call=None):
         '''Internal response generator, no sense in repeating the same
         code twice, right? ;)
         '''
         myargs = {}
         method = method.lower()
 
+        params = params or {}
+
         if method == 'get':
             url = '%s?%s' % (url, urllib.urlencode(params))
         else:
             myargs = params
 
         func = getattr(self.client, method)
-        response = func(url, data=myargs, auth=self.auth)
+        response = func(url, data=myargs, files=files, headers=self.headers, auth=self.auth)
         content = response.content.decode('utf-8')
 
         # create stash for last function intel
@@ -228,16 +217,19 @@ class Twython(object):
             raise TwythonError('Response was not valid JSON, unable to decode.')
 
         if response.status_code > 304:
-            # Just incase there is no error message, let's set a default
-            error_msg = 'An error occurred processing your request.'
-            if content.get('error') is not None:
-                error_msg = content['error']
-
+            # If there is no error message, use a default.
+            error_msg = content.get(
+                'error', 'An error occurred processing your request.')
             self._last_call['api_error'] = error_msg
 
-            raise TwythonError(error_msg,
-                               error_code=response.status_code,
-                               retry_after=response.headers.get('retry-after'))
+            if response.status_code == 420:
+                exceptionType = TwythonRateLimitError
+            else:
+                exceptionType = TwythonError
+
+            raise exceptionType(error_msg,
+                                error_code=response.status_code,
+                                retry_after=response.headers.get('retry-after'))
 
         return content
 
@@ -248,31 +240,23 @@ class Twython(object):
     we haven't gotten around to putting it in Twython yet. :)
     '''
 
-    def request(self, endpoint, method='GET', params=None, version=1):
-        params = params or {}
-
+    def request(self, endpoint, method='GET', params=None, files=None, version=1):
         # In case they want to pass a full Twitter URL
-        # i.e. http://search.twitter.com/
+        # i.e. https://search.twitter.com/
         if endpoint.startswith('http://') or endpoint.startswith('https://'):
             url = endpoint
         else:
             url = '%s/%s.json' % (self.api_url % version, endpoint)
 
-        content = self._request(url, method=method, params=params, api_call=url)
+        content = self._request(url, method=method, params=params, files=files, api_call=url)
 
         return content
 
     def get(self, endpoint, params=None, version=1):
-        params = params or {}
         return self.request(endpoint, params=params, version=version)
 
-    def post(self, endpoint, params=None, version=1):
-        params = params or {}
-        return self.request(endpoint, 'POST', params=params, version=version)
-
-    def delete(self, endpoint, params=None, version=1):
-        params = params or {}
-        return self.request(endpoint, 'DELETE', params=params, version=version)
+    def post(self, endpoint, params=None, files=None, version=1):
+        return self.request(endpoint, 'POST', params=params, files=files, version=version)
 
     # End Dynamic Request Methods
 
@@ -329,7 +313,7 @@ class Twython(object):
     def get_authorized_tokens(self):
         """Returns authorized tokens after they go through the auth_url phase.
         """
-        response = self.client.get(self.access_token_url, auth=self.auth)
+        response = self.client.get(self.access_token_url, headers=self.headers, auth=self.auth)
         authorized_tokens = dict(parse_qsl(response.content))
         if not authorized_tokens:
             raise TwythonError('Unable to decode authorized tokens.')
@@ -362,40 +346,12 @@ class Twython(object):
         if request.status_code in [301, 201, 200]:
             return request.text
         else:
-            raise TwythonError('shortenURL() failed with a %s error code.' % request.status_code, request.status_code)
+            raise TwythonError('shortenURL() failed with a %s error code.' % request.status_code)
 
     @staticmethod
     def constructApiURL(base_url, params):
         return base_url + "?" + "&".join(["%s=%s" % (Twython.unicode2utf8(key), urllib.quote_plus(Twython.unicode2utf8(value))) for (key, value) in params.iteritems()])
 
-    def bulkUserLookup(self, ids=None, screen_names=None, version=1, **kwargs):
-        """ A method to do bulk user lookups against the Twitter API.
-
-            Documentation: https://dev.twitter.com/docs/api/1/get/users/lookup
-
-            :ids or screen_names: (required)
-            :param ids: (optional) A list of integers of Twitter User IDs
-            :param screen_names: (optional) A list of strings of Twitter Screen Names
-
-            :param include_entities: (optional) When set to either true, t or 1,
-                                     each tweet will include a node called
-                                     "entities,". This node offers a variety of
-                                     metadata about the tweet in a discreet structure
-
-            e.g x.bulkUserLookup(screen_names=['ryanmcgrath', 'mikehelmick'],
-                                 include_entities=1)
-        """
-        if ids is None and screen_names is None:
-            raise TwythonError('Please supply either a list of ids or \
-                                screen_names for this method.')
-
-        if ids is not None:
-            kwargs['user_id'] = ','.join(map(str, ids))
-        if screen_names is not None:
-            kwargs['screen_name'] = ','.join(screen_names)
-
-        return self.get('users/lookup', params=kwargs, version=version)
-
     def search(self, **kwargs):
         """ Returns tweets that match a specified query.
 
@@ -427,7 +383,7 @@ class Twython(object):
             e.g x.search(q='jjndf', page='2')
         """
         if 'q' in kwargs:
-            kwargs['q'] = urllib.quote_plus(Twython.unicode2utf8(kwargs['q']))
+            kwargs['q'] = urllib.quote_plus(Twython.unicode2utf8(kwargs['q'], ':'))
 
         return self.get('https://search.twitter.com/search.json', params=kwargs)
 
@@ -480,7 +436,17 @@ class Twython(object):
         return self._media_update(url,
                                   {'image': (file_, open(file_, 'rb'))},
                                   params={'tile': tile})
-
+    
+    def bulkUserLookup(self, **kwargs):
+        """Stub for a method that has been deprecated, kept for now to raise errors
+            properly if people are relying on this (which they are...).
+        """
+        warnings.warn(
+            "This function has been deprecated. Please migrate to .lookupUser() - params should be the same.",
+            DeprecationWarning,
+            stacklevel=2
+        )
+    
     def updateProfileImage(self, file_, version=1):
         """Updates the authenticating user's profile image (avatar).
 
@@ -509,44 +475,7 @@ class Twython(object):
                                   **params)
 
     def _media_update(self, url, file_, params=None):
-        params = params or {}
-        oauth_params = {
-            'oauth_timestamp': int(time.time()),
-        }
-
-        #create a fake request with your upload url and parameters
-        faux_req = oauth.Request(method='POST', url=url, parameters=oauth_params)
-
-        #sign the fake request.
-        signature_method = oauth.SignatureMethod_HMAC_SHA1()
-
-        class dotdict(dict):
-            """
-            This is a helper func. because python-oauth2 wants a
-            dict in dot notation.
-            """
-
-            def __getattr__(self, attr):
-                return self.get(attr, None)
-            __setattr__ = dict.__setitem__
-            __delattr__ = dict.__delitem__
-
-        consumer = {
-            'key': self.app_key,
-            'secret': self.app_secret
-        }
-        token = {
-            'key': self.oauth_token,
-            'secret': self.oauth_secret
-        }
-
-        faux_req.sign_request(signature_method, dotdict(consumer), dotdict(token))
-
-        #create a dict out of the fake request signed params
-        self.headers.update(faux_req.to_header())
-
-        req = requests.post(url, data=params, files=file_, headers=self.headers)
-        return req.content
+        return self.post(url, params=params, files=file_)
 
     def getProfileImageUrl(self, username, size='normal', version=1):
         """Gets the URL for the user's profile image.
@@ -621,7 +550,10 @@ class Twython(object):
 
         for line in stream.iter_lines():
             if line:
-                callback(simplejson.loads(line))
+                try:
+                    callback(simplejson.loads(line))
+                except ValueError:
+                    raise TwythonError('Response was not valid JSON, unable to decode.')
 
     @staticmethod
     def unicode2utf8(text):
